#!/usr/bin/perl -w
use strict;
#use diagnostics;

use lib "$ENV{CMISS_EXAMPLES}/common/cmiss_type1";
use test_cmiss;

use File::Copy;

if (@ARGV < 1)
  {
	 print "Usage: $0 old_examples new_example_repository example_path\n";
	 exit;
  }

#Where all the examples are stored on disk i.e. /product/cmiss/examples
my $old_examples = $ARGV[0];
#Where all the examples are stored on disk i.e. /product/cmiss/examples
my $example_location = $ARGV[1];
my $example_path = "";
if (@ARGV > 2)
  {
    $example_path = $ARGV[2];
  }

#Script/template path
my $template_path = "$example_location/common/cmiss_type1";
my $readdir_grep_string = q/^[^\.]/;

if (! -d $old_examples)
  {
    die "Example directory $old_examples not found.";
  }


#Recurse down tree.
sub transfer_tree
  {
    my $old_examples = shift;
    my $example_location = shift;
    my $example_path = shift;
    my $old_example_dir = $old_examples . "/" . $example_path;
    my $example_dir = $example_location . "/" . $example_path;
    my @subdirectories;
    my $command;
    my @filelist;
    my $made_testing_dir = 0;
    
    if (-f "$old_example_dir/name.txt")
      {
	print "Transfer in $example_dir\n";
	if (! -d "$example_dir")
	  {
	    mkdir("$example_dir", 0755)
	      || die "Couldn't make $example_dir directory\n";
	  }
	if (! -d "$example_dir/web_data")
	  {
	    mkdir("$example_dir/web_data", 0755)
	      || die "Couldn't make $example_dir/web_data directory\n";
	  }
	if (! -d "$example_dir/control")
	  {
	    mkdir("$example_dir/control", 0755)
	      || die "Couldn't make $example_dir/control directory\n";
	  }
	if (! -d "$example_dir/cmiss_input")
	  {
	    mkdir("$example_dir/cmiss_input", 0755)
	      || die "Couldn't make $example_dir/cmiss_input directory\n";
	  }
	
	#Process the main directory
	opendir(DIR, "$old_example_dir") || die "can't opendir $old_example_dir: $!\n";
	@filelist = grep { /$readdir_grep_string/ } readdir(DIR);
	closedir DIR;
	for (@filelist)
	  {
	    if (! -d "$old_example_dir/$_")
	      {
		if (m%^(description.html|name.txt)$%)
		  {
		    system ("cp $old_example_dir/$_ $example_dir/web_data");
		  }
		else
		  {
		    system ("cp $old_example_dir/$_ $example_dir/cmiss_input");
		  }
	      }
	    else
	      {
		if (m%^images$%)
		  {
		    system ("cp -r $old_example_dir/$_ $example_dir/web_data");
		  }
	      }
	  }
	
	#Process the generated directory
	opendir(DIR, "$old_example_dir/generated") || die "can't opendir $old_example_dir/generated: $!\n";
	@filelist = grep { /$readdir_grep_string/ } readdir(DIR);
	closedir DIR;

	if (-d "$example_dir/testing")
	  {
	    $made_testing_dir = 1;
	  }

	for (@filelist)
	  {
	    if (! -d "$old_example_dir/generated/$_")
	      {
		if (m%^(.*)_fixed$%)
		  {
		    if (! $made_testing_dir)
		      {
			mkdir("$example_dir/testing", 0755)
			  || die "Couldn't make $example_dir/testing directory\n";
			mkdir("$example_dir/testing/diff", 0755)
			  || die "Couldn't make $example_dir/testing/diff directory\n";
			mkdir("$example_dir/testing/diff/generic", 0755)
			  || die "Couldn't make $example_dir/testing/diff/generic directory\n";
			mkdir("$example_dir/testing/retain", 0755)
			  || die "Couldn't make $example_dir/testing/retain directory\n";
			mkdir("$example_dir/testing/retain/generic", 0755)
			  || die "Couldn't make $example_dir/testing/retain/generic directory\n";
			#Make a dummy file so that the log gets copied.
			system ("echo > $example_dir/testing/retain/generic/cmiss_test.log");
			$made_testing_dir = 1;
		      }
		    system ("cp $old_example_dir/generated/$_ $example_dir/testing/diff/generic/$1");
		  }
	      }
	  }
	
	#Add the default cm controls
	if (-f "$example_dir/cmiss_input/test_output.com")
	  {
	    system ("cp -r $example_location/template/control $example_dir");
	  }
	else
	  {
	    system ("cp -r $example_location/template_notest/control $example_dir");
	  }
      }
    else
      {
	#Look further down.
	
	opendir(DIR, "$old_example_dir") || die "can't opendir $old_example_dir: $!\n";
	@subdirectories = grep { /$readdir_grep_string/ } readdir(DIR);
	closedir DIR;
	
	if (! -d "$example_dir")
	  {
	    mkdir("$example_dir", 0755)
	      || die "Couldn't make $example_dir directory\n";
	  }
	
	for (@subdirectories)
	  {
	    if (-d "$old_example_dir/$_")
	      {
		if (m%^generated$%)
		  {
		    #Ignore
		  }
		else
		  {
		    if (length ($example_path) > 0)
		      {
			transfer_tree($old_examples, $example_location, $example_path . "/" . $_);
		      }
		    else
		      {
			transfer_tree($old_examples, $example_location, $_);
		      }
		  }
	      }
	    else
	      {
		if (m%^(description.html|node_name.txt)$%)
		  {
		    if (! -d "$example_dir/web_data")
		      {
			mkdir("$example_dir/web_data", 0755)
			  || die "Couldn't make $example_dir/web_data directory\n";
		      }
		    copy("$old_example_dir/$_", "$example_dir/web_data");
		  }
	      }
	  }
      }
  }

transfer_tree($old_examples, $example_location, $example_path);

