eval 'exec perl -S $0 ${1+"$@"}'
    if 0;

use strict;
#use diagnostics;

$0 =~ s%^.*/%%; # remove path from warning messages
BEGIN {
    $^W = 1; # turn on warnings
    $SIG{__WARN__} = sub { warn "$0 warning: ",@_ };
}
sub dirname ($) {
    # return the directory part of a filename
    $_[0] =~ m'^(.*)/[^/]*$' ? $1 : '.';
}
# use modules in the same directory as this file if they exist.
use lib dirname __FILE__, "$ENV{CMISS_EXAMPLES}/common/cmiss_type1";
use test_cmiss;

my %options;
my $usage = "Usage: $0 [-h] -e example\n";
my $example_path = "";

if (@ARGV < 1)
  {
    print $usage;
    exit;
  }
if (!(test_cmiss::getopts('he:', \%options)))
  {
    print $usage;
    exit;
  }

for (keys %options)
  {
    if (m%^h$%)
      {
	 print $usage;
	 exit;
      }
    elsif (m%^e$%)
      {
	#The relative path to this example i.e. a/a1
	$example_path = $options{$_};
      }
  }

if (! $example_path)
  {
    print "Example option missing\n";
    print $usage;
    exit;
  }

#Where all the examples are stored on disk i.e. /product/cmiss/examples
my $example_location = $ENV{CMISS_EXAMPLES};
#Script/template path
my $template_path = "$example_location/common/cmiss_type1";
#Template file
my $template_file = "$template_path/template.html";
my $example_dir = $example_location . "/" . $example_path;
my $example_name = substr ($example_dir, rindex ($example_dir, '/') + 1);
my $expanded_name = substr($example_name, 0, 1);
my $i;
for ($i = 1 ; $i < length($example_name) ; $i++)
  {
     $expanded_name = $expanded_name . "/" . substr($example_name, 0, $i + 1);
  }
my $example_comfile = ();
if (-f "$example_dir/cmiss_input/example_$example_name.com")
  {
     $example_comfile = "example_$example_name"
  }
elsif (-f "$example_dir/cmiss_input/$example_name.com")
  {
     $example_comfile = "$example_name"
  }
else
  {
     $example_comfile = ();
  }
if ($expanded_name ne $example_path)
  {
     #The old name mangling doesn't work so use the full path
     $example_name = $example_path;
  }

my $generated_comment = "<!-- Generated by $0 -->";

my $example_tree_list =
  do {
    my $list = '';
    my @subpath_tokens = split "/", $example_path;
    # Prepend the help page if we know where it is.
    if ( exists $ENV{CMISS_WWW_URL} )
      {
	$list .= <<HTML;
<a href="$ENV{CMISS_WWW_URL}help/index.html">CMISS Help</a> /
HTML
	unshift @subpath_tokens, 'Examples'
      }
    else
      {
	unshift @subpath_tokens, 'CMISS Examples'
      }
    my $example_name = pop @subpath_tokens;
    # References to parent directories
    while ( @subpath_tokens )
      {
	my $dots = '../' x @subpath_tokens;
	my $token = shift @subpath_tokens;
	$list .= <<HTML;
<a href="${dots}index.html">$token</a> /
HTML
      }
    $list .= $example_name;
    $list;
  };

my $iffalse = 0;
my $ifdepth = 0;
my $format;
my $output;
my $filetime;
my @filestat;
my $readdir_grep_string = q/^(\..*|CVS|.*~)$/;
my $example_lock_file = "$example_dir/lock";
my $test_summary = "test.summary";
my $test_log = "log";

my $example_fullname = do
  {
    if (-f "$example_dir/web_data/name.txt")
      {
	open (NAMEFILE, "<$example_dir/web_data/name.txt")
	  || die "$0: can't open $example_dir/web_data/name.txt: $!\n";
	my $name = <NAMEFILE>;
	close NAMEFILE;
	$name
      }
    else
      {
	$example_dir;
      }
  };

#Construct an expiration date that lasts until tomorrow morning.
#I add 28800s = 8hours so that it always goes into tomorrow after 4am each day
my @gm_time = gmtime(time() + 28800);
my $gm_weekday = (qw(Sun Mon Tue Wed Thu Fri Sat))[$gm_time[6]];
my $gm_month = (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec))[$gm_time[4]];
my $gm_year = 1900 + $gm_time[5];
#The expiration time is 6am NZ STD time which is before everyone is 
#looking at them.  Unfortunately this does not do the required
#gymnastics so that it could be 7am and change with daylight savings
my $expiration = "$gm_weekday, $gm_time[3] $gm_month $gm_year 18:00:00 GMT";

#Get environment variables, use a sensible but failing dummy if not found.
my $cmiss_www_example_url = "/\$CMISS_WWW_EXAMPLE_URL";
if ($ENV{CMISS_WWW_EXAMPLE_URL})
  {
     $cmiss_www_example_url = $ENV{CMISS_WWW_EXAMPLE_URL};
  }
my $cmiss_www_url = "/\$CMISS_WWW_URL";
if ($ENV{CMISS_WWW_URL})
  {
     $cmiss_www_url = $ENV{CMISS_WWW_URL};
  }

sub abort
  {
      # Die but make sure we don't leave the lock file lying around.
    test_cmiss::abort_locks();
    die "$0: ",@_;
  }

$SIG{__DIE__} = \&abort;
$SIG{INT} = sub
  {
    warn "interrupt received\n";
    die "aborted\n";
  };
$SIG{TERM} = sub
  {
    warn "SIGTERM received\n";
    die "aborted\n"
  };

sub human_size ($) {
  my ($size) = @_;
  # Converts a number of bytes to a pretty string in k or M.
  defined $size or return '- ';
  $size < 1000 and return sprintf '%d ', $size;
  $size /= 1024;
  $size < 10 and return sprintf '%1.1fk', $size;
  $size < 1000 and return sprintf '%dk', $size;
  $size /= 1024;
  $size < 10 and return sprintf '%1.1fM', $size;
  return sprintf '%dM', $size;
}

sub human_date ($) {
  my ($time) = @_;
  # Converts a time to a pretty string DD-MMM-YYYY.
  my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
  my ($mday,$mon,$year) = (localtime $time)[3,4,5];
  return sprintf '%02d-%s-%04d', $mday, $months[$mon], $year + 1900;
}

sub max (@)
  {
    @_ or return;
    my $max = shift @_;
    foreach (@_)
      {
	$_ < $max or $max = $_;
      }
    return $max;
  }

sub html_index_files ($$@)
  # returns a string containing an html listing of files in a directory.
  {
    my ($prefix,$path,@names) = @_;

    @names or return '';

    $path = $path eq '.' ? '' : "$path/";

    my @data = map
      {
	my $name = $_;
	my $file = "$path$name";
	my $link = "$prefix$name";
	stat $file or die "can't stat $file: $!\n";
	my ($size,$mtime) = (stat _)[7,9];
	if( -d $file )
	  { # directory
	    $name .= '/';
	    $link .= '/index.html';
	    $size = undef;
	  }
	{ name => $name,
	    link => $link,
	      modified => human_date $mtime,
		size => human_size $size }
      } @names;
    undef @names;

    my %headings = ( name => 'Name', size => 'Size', modified => 'Modified' );

    # find the longest item in each column.
    my %widths =
      map
	{
	  my $key = $_;
	  ($key => max map { length $_->{$key} } (\%headings,@data))
	} keys %headings;

    my ($heading, @lines) =
      map
	{
	  my ($name,$link,$modified,$size) =
	    @{$_}{qw(name link modified size)};

	  (defined $link ? "<a href=\"$link\">$name</a>" : $name) .
	    ' ' x (2 + $widths{name} - length $name) .
	      $modified .
		' ' x (2 + $widths{modified} - (length $modified)
			 + $widths{size} - length $size) .
		    $size
	} (\%headings,@data);

    undef %headings;
    undef @data;
    undef %widths;

    return join "\n", '<pre>',$heading,'<hr />',@lines,"</pre>\n";
}

{
  my $login = getlogin || 'unknown';
  my $program = $0; $program =~ s|^.*/||;

  test_cmiss::start_lock( "$example_lock_file",
			  "$login.$$." . test_cmiss::hostname . ":$program" );
}

# Get list of cmiss_input files.
my @inputdir;
if( -d "$example_dir/cmiss_input" )
  {
    opendir(DIR, "$example_dir/cmiss_input") || die "can't opendir $example_dir/cmiss_input: $!\n";
# List all files except for hidden ones.
    @inputdir = grep { ! /$readdir_grep_string|^index\.html/o } readdir(DIR);
    closedir DIR;

    # Provide html indices for subdirectories of the cmiss_input directory in
    # case the web server doesn't automatically generate directory listings.
    test_cmiss::recurse_directory_tree
      sub
	{
	  my ($path,$subdirs,$files) = @_;

	  my $index = "$path/index.html";
	  my $magic_line = "<html>$generated_comment\n";

	  if( -s $index )
	    { # Index already exists and has contents:
	      # don't overwrite if written by someone else.
	      open FILE, $index or die "can't open $index: $!\n";
	      my $line1 = <FILE>;
	      close FILE;
	      if( $line1 ne $magic_line)
		{ # Index not generated by this script: skip subdirs
		  @$subdirs = ();
		  return;
		}
	    }

	  open FILE, "> $index" or die "can't open > $index: $!\n";
	  print FILE $magic_line;

	  # Get path within example.
	  my $subpath = $path;
	  $subpath =~ s%^\Q$example_dir\E/%%;

	  print FILE <<EOF;
<title>$example_name/$subpath</title>
<h1>Example $example_name: Directory listing of $subpath</h1>
<hr />
EOF

	  # prune list to those that should be displayed
	  @$subdirs = grep { ! /$readdir_grep_string/o } @$subdirs;
	  @$files = grep { ! /$readdir_grep_string|^index\.html$/o } @$files;
	  print FILE
	    html_index_files( '',$path,
			      ( '..', sort (@$subdirs,@$files) ) );

	  print FILE "<hr />\n",
	    "Listing generated ",scalar localtime time, "\n";
	  print FILE "</html>\n";

	  truncate FILE, tell FILE or die "can't truncate $index: $!\n";
	  close FILE or die "error closing $index: $!\n";
	},
	sub
	  {
# 	    my ($path,$results,$files) = @_;
	    1
	  },
	 "$example_dir/cmiss_input";
  }

# Get list of web_data files.
opendir(DIR, "$example_dir/web_data") || die "can't opendir $example_dir/web_data: $!\n";
# List all files except for hidden ones.
my @webdatadir = grep { ! /$readdir_grep_string/ } readdir(DIR);
closedir DIR;

# Get list of tested versions.
my @outputdir = ();
if (-d "$example_dir/output")
  {
     #Recursively look for subdirectories and replace each file with the subdirs so that
     #we are working on the leaves of the tree.
     test_cmiss::add_leaf_directories(\@outputdir, "$example_dir/output", "");
     @outputdir = sort { $a cmp $b } @outputdir;
  }

open (FILE, ">$example_dir/index.html") || die "can't open $example_dir/index.html: $!\n";

open (INPUT, "<$template_file") || die "can't open $template_file: $!\n";

while(<INPUT>)
  {
     s/\$example_name/$example_name/g;
     s/<EXAMPLE_TREE_LIST>/$example_tree_list/;
     
     s/\$expiration/$expiration/g;
     if ($example_comfile)
	{
	  s/\$example_comfile/$example_comfile/g;
	}
     else
	{
	  s/\$example_comfile/"No comfile found"/g;
	}
     s%\$\{CMISS_WWW_EXAMPLE_URL\}%$cmiss_www_example_url%g;
     s%\$\{CMISS_HELPURL\}%${cmiss_www_url}help/%g;
    s%\$\{CMISS_WWW_URL\}%$cmiss_www_url%g;

     #All the other tags and tokens work on a line by line basis.
     #Therefore an <if IMAGES> tag should be on a line by itself
     #as well as the corresponding </if>

     #Allow conditional sections in the template.
     #<if *> is only recognised when blocked against the left margin.
     #i.e. <if IMAGES>
     #     This is only displayed if the images directory exists.
    #     </if>
     if (m%^<if [^>]*>%)
	{
	  $iffalse = 0;
	  if (m%^<if IMAGES>%)
	     {
		if ( ! -d "$example_dir/web_data/images" )
		  {
		     $iffalse = 1;
		  }
	     }
	  elsif (m%^<if CMTESTING>%)
	     {
		if ( ! -f "$example_dir/cmiss_input/test_output.com" )
		  {
		     $iffalse = 1;
		  }
	     }
	  elsif (m%^<if TESTING>%)
	     {
		unless( @outputdir )
		  {
		     $iffalse = 1;
		  }
	     }
	  if ($iffalse)
	     {
		$ifdepth = 1;
		while ($ifdepth && ($_ = <INPUT>))
		  {
		     if (m%^<if %)
			{
			  $ifdepth++;
			}
		     elsif (m%^</if%)
			{
			  $ifdepth--;
			}
		  }
	     }
	}
     elsif (m%^</if[^>]*>%)
	{
	  # Discard
	}

     #Substitute strings for content
     elsif (m/\$name_file_goes_here/)
	{
	  print FILE "$example_fullname\n";
	}
     elsif (m/\$comfile_file_goes_here/)
	{
	  if ($example_comfile) {
	      open (COMFILE, "<$example_dir/cmiss_input/$example_comfile.com")
		  || die "can't open $example_dir/cmiss_input/$example_comfile.com: $!\n";
	      #print FILE <COMFILE>;
	      # Check for special characters that need to 
	      # be escaped
	      while (<COMFILE>)
	      {
		  s/</&lt;/g;
		  s/>/&gt;/g;
		  print FILE $_;
	      }
	      close COMFILE;
	  } else {
	      print FILE "**** Comfile not found! ****\n";
	  }
	}
     elsif (m/\$test_output_file_goes_here/)
	{
#	  open (COMFILE, "<$template_path/test_pre_output.com")
#		 || die "can't open $template_path/test_pre_output.com: $!\n";
#	  print FILE <COMFILE>;
#	  close COMFILE;
	  open (COMFILE, "<$example_dir/cmiss_input/test_output.com")
	     || die "can't open $example_dir/cmiss_input/test_output.com: $!\n";
	  print FILE <COMFILE>;
	  close COMFILE;
#	  open (COMFILE, "<$template_path/test_post_output.com")
#		 || die "can't open $template_path/test_post_output.com: $!\n";
#	  print FILE <COMFILE>;
#	  close COMFILE;
	}
     elsif (m/\$description_file_goes_here/)
	{
	  if (-f "$example_dir/web_data/description.html")
	     {
		open (DESCRIPTION_FILE, "<$example_dir/web_data/description.html")
		  || die "can't open $example_dir/web_data/description.html: $!\n";
		while (<DESCRIPTION_FILE>)
		  {
		     s%\$\{CMISS_WWW_EXAMPLE_URL\}%$cmiss_www_example_url%g;
		     s%\$\{CMISS_HELPURL\}%${cmiss_www_url}help/%g;
		    s%\$\{CMISS_WWW_URL\}%$cmiss_www_url%g;
		     print FILE;
		  }
		close DESCRIPTION_FILE;
	     }
	}
     # The format for the files in the file list is expected following the tag.
     elsif (m/\$file_list_file_goes_here/)
	{
	  print FILE
	    html_index_files( 'cmiss_input/',"$example_dir/cmiss_input",
			      sort input_file_sort_function @inputdir );
	}
     # The format for the files in the file list is expected following the tag.
     elsif (m/\$version_testing_list_goes_here/)
	{
	  my $current_version_group = ".";
	  $format = substr ($_, length ("\$version_testing_list_goes_here"));
	  if (@outputdir)
	  {
	      # create title row for the table
	      $output = $format;
	      $output =~ s%\$version% %g;
	      $output =~ s%\$status%Status%g;
	      $output =~ s%\$time_stamp%Tested%g;
	      $output =~ s%\$clock_time%Real time (s)%g;
	      print FILE $output;
	  }
	  foreach my $version (@outputdir)
	     {
		my $result_path = "$example_dir/output/$version";
		my $summary_file = "$result_path/$test_summary";
		my $status = "Incomplete";
		my $execution_time = "unknown";
		my $time_stamp;
		my $version_group = dirname($version);
		my $version_filename = &filename($version);
		if ($version_group && ($version_group ne $current_version_group))
		  {
		    $current_version_group = $version_group;

		    $output = $format;
		    $output =~ s%\$version%$version_group%g;
		    $output =~ s%\$status%%g;
		    $output =~ s%\$time_stamp%%g;
		    $output =~ s%\$clock_time%%g;
		    print FILE $output;
		  }
		$output = $format;
		if( -f $summary_file )
		  {
		     stat $summary_file or
			 die "can't stat $summary_file: $!\n";
		     $time_stamp = localtime( (stat _)[9] );
		     open TEST_FILE, $summary_file
			or die "can't open $summary_file: $!\n";

		     my $line;
		     if( ! defined( $line = <TEST_FILE> )
or $line !~ /^\Q*****\E CMISS_testing(?: \S+){3} (\S+) \Q*****\E\n/ )
		     {
			 warn "header of $summary_file not recognized\n";
		     }
		     else {
			 $status = $1;
			 if( ! defined( $line = <TEST_FILE> )
or $line !~ /^Clock_execution (\S+) seconds/ )
			 {
			     warn "timing in $summary_file not understood\n";
			 }
			 else {
			     $execution_time = $1;
			 }
		     }
		     close TEST_FILE;
		  }
		else
		  {
		     stat $result_path or
			 die "can't stat $result_path: $!\n";
		     $time_stamp = localtime( (stat _)[9] );
		  }

		my $success;
		if( $status =~ /^success/i ) {
		    $success = 1;
		} else {
		    $status = "<b><FONT COLOR=\"#FF0000\">$status</FONT></b>";
		}

		$output =~ s%\$version%<A HREF = \"output/$version/$test_log\">$version_filename</A>%g;
		$output =~ s%\$status%<A HREF = \"output/$version/$test_summary\">$status</A>%g;
		$output =~ s%\$time_stamp%$time_stamp%g;
		$output =~ s%\$clock_time%$execution_time%g;
		print FILE $output;

		unless( $success ) {
		    # write last break and last success
		    my ($date,$duration);
#  		    my $info_ref = test_cmiss::last_break_and_success $result_path;
		    my ($last_break_ref,$last_success_ref) = @{
			test_cmiss::last_break_and_success $result_path
			}{'break','success'};
		    if( defined $last_break_ref ) {
			my $time_stamp =
			    localtime($last_break_ref->{'start time'});
			my $duration = $last_break_ref->{'real time'};
			$output = $format;
			$output =~ s%\$version% %g;
			$output =~ s%\$status%last break%g;
			$output =~ s%\$time_stamp%$time_stamp%g;
			$output =~ s%\$clock_time%$duration%g;
			print FILE $output;
		    }
		    if( defined $last_success_ref ) {
			my $time_stamp =
			    localtime($last_success_ref->{'start time'});
			my $duration = $last_success_ref->{'real time'};
			$output = $format;
			$output =~ s%\$version% %g;
			$output =~ s%\$status%last success%g;
			$output =~ s%\$time_stamp%$time_stamp%g;
			$output =~ s%\$clock_time%$duration%g;
			print FILE $output;
		    }
		}
	     }
	}
     elsif (m/\$file_testing_list_goes_here/)
     {
	 if (-d "$example_dir/output")
	 {
	     #Create a hash of arrays that lists each file and the versions
	     #it was tested in.
	     my %files_tested = ();
	     foreach my $version (@outputdir) {
		 my $version_dir = "$example_dir/output/$version";
		 if (-d "$version_dir") {
		     test_cmiss::start_lock_output $version_dir;

		     opendir(DIR, "$version_dir") || die "can't opendir $version_dir: $!\n";
		     # List all files except for hidden ones.
		     my @output_files =
			 grep { ! (m/$readdir_grep_string/
				   || $_ eq $test_summary
				   || $_ eq $test_log ) } readdir(DIR);
		     closedir DIR;

		     foreach my $test_file (@output_files) {
			 my $full_filename = "$version_dir/$test_file";
			 # Get the test header and extract the tokens
			 open (TEST_FILE, $full_filename)
			     || die "can't open $full_filename: $!\n";
			 my $header = <TEST_FILE>;
			 close TEST_FILE;
			 my %file_details = ('test_file' => $test_file,
					     'version' => $version);
			 my $answer_file;
			 if( defined $header and
			     $header =~ m/^\*{5} CMISS_testing/ ) {
			     my @header = split / /, $header;
			     $answer_file = $header[2];
			     @file_details{'test_type','compare_version',
					   'status'} = @header[3..5];
			 } else {
			     $answer_file = $test_file;
			 }
			 push @{$files_tested{$answer_file}},\%file_details;
		     }
		     test_cmiss::end_lock_output $version_dir;
		 }
	     }
	     print FILE "<UL>\n";
	     foreach my $answer_file (sort output_file_sort_function
				      keys %files_tested) {
                 my $current_version_group = ".";
		 print FILE "<LI>$answer_file\n";
		 print FILE "<br><TABLE>\n";
		 foreach my $file_details_ref (@{$files_tested{$answer_file}})
		 {
		     my ($test_file,$version,$test_type,
			 $compare_version,$status) =
			     @{$file_details_ref}{'test_file','version',
						  'test_type',
						  'compare_version','status'};
		     my $version_group = dirname($version);
		     my $version_filename = &filename($version);
		     if ($version_group && ($version_group ne $current_version_group))
		       {
			 $current_version_group = $version_group;
			 print FILE "<TR><TD><b>$version_group</b></TD><TD></TD><TD></TD><TD></TD></TR>\n";
		       }

		     print FILE "<TR>";
		     if( exists $file_details_ref->{test_type} ) {
			 my $success = $status =~ m/^Success$/i;
			 if( $success ) {
			     print FILE "<TD>$status</TD>";
			 } else {
			     print FILE
"<TD><b><FONT COLOR=\"#FF0000\">$status</FONT></b></TD>";
			 }
			 print FILE "<TD><b>$version_filename:</b></TD>";
			 if ($status =~ m/^Missing$/i) {
			     print FILE <<HTML;
<TD><b>output file not generated for $test_type</b>;
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A>.</TD>
HTML
			 } elsif( $test_type eq 'diff' ) {
			     if ($success) {
				 print FILE <<HTML;
<TD>diff test: no differences with
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A>.</TD>
HTML
                             } else {
				 print FILE <<HTML;
<TD>diff test:
<b><A HREF = \"output/$version/$test_file\">differences</A> with
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A></b>;
<A HREF = \"/cgi-bin/Example_diff_reconstruct.cgi?answer=testing/diff/$compare_version/$answer_file&diff=output/$version/$test_file\">Test output</A>.
</TD>
HTML
			     }
			 } elsif( $test_type eq 'ndiff' ) {
			     if( $success ) {
				 print FILE <<HTML;
<TD>ndiff test: no significant differences with
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A>.
</TD>
HTML
			     } else {
				 print FILE <<HTML;
<TD>ndiff test:
<b><A HREF = \"output/$version/$test_file\">significant differences</A> with
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A></b>.
</TD>
HTML
			     }
			 } elsif( $test_type eq 'retain' ) {
			     print FILE <<HTML;
<td><a href=\"output/$version/$test_file\">$test_file</a>. </td>
HTML
                         } elsif( $test_type eq 'exit' &&
				  $answer_file eq 'signal') {
			     print FILE <<HTML;
<td><b>exit due to $status signal</b>.</td>
HTML
                         } elsif( $test_type eq 'exit' &&
				  $answer_file eq 'status') {
			     print FILE <<HTML;
<td><b>error exit status $status</b>.</td>
HTML
                         } else {
			     print FILE <<HTML;
<TD>$test_type test:
<A HREF = \"output/$version/$test_file\">Test output</A>;
<A HREF = \"testing/$test_type/$compare_version/$answer_file\">$compare_version answer</A>.
</TD>
HTML
                         }
		     } else { # no test type
			 print FILE <<HTML;
<td></td>
<td><b>$version:</b></td>
<td>kept file <a href=\"output/$version/$test_file\">$answer_file</a>. </td>
HTML
                     }
		     print FILE "</TR>\n";
		 }
		 print FILE "</TABLE><br></LI>\n";
	     }
	     print FILE "</UL>\n";
	 }
     }
     elsif (m/\$generated_time_stamp/)
	{
	  $output = localtime(time());
	  s/\$generated_time_stamp/$output/;
	  print FILE $_;
	}
     elsif (m/\$input_modified_time_stamp/)
	{
	  # Look through the cmiss_input files and the web_data files
	  # for the most recent
	  $filetime = 0;
	  for (@inputdir)
	     {
		@filestat = stat "$example_dir/cmiss_input/$_";
		if ($filestat[9] > $filetime)
		  {
		     $filetime = $filestat[9];
		  }
	     }
	  for (@webdatadir)
	     {
		@filestat = stat "$example_dir/web_data/$_";
		if ($filestat[9] > $filetime)
		  {
		     $filetime = $filestat[9];
		  }
	     }
	  $output = localtime($filetime);
	  s/\$input_modified_time_stamp/$output/;
	  print FILE $_;
	}
     else
	{
	  print FILE $_;
	}
  }

close INPUT;
close FILE;

test_cmiss::end_lock("$example_lock_file");

sub output_file_sort_function
  {
    if ($a eq 'signal')
      {
	return (-1);
      }
    if ($b eq 'signal')
      {
	return (1);
      }
    if ($a eq 'status')
      {
	return (-1);
      }
    if ($b eq 'status')
      {
	return (1);
      }
    if ($a =~ m/^cmiss_test.log$/)
      {
	return (-1);
      }
    if ($b =~ m/^cmiss_test.log$/)
      {
	return (1);
      }
    if ($a =~ m/^cmiss_test.out$/)
      {
	return (-1);
      }
    if ($b =~ m/^cmiss_test.out$/)
      {
	return (1);
      }
    return ($a cmp $b);

  }

sub input_file_sort_function
  {
    if ($example_comfile)
      {
	if ($a =~ m/^$example_comfile.com$/)
	  {
	    return (-1);
	  }
	if ($b =~ m/^$example_comfile.com$/)
	  {
	    return (1);
	  }
      }
    return ($a cmp $b);
  }

sub filename ($) {
    # return the non directory part of a filename
    $_[0] =~ m'([^/]*)$' ? $1 : '';
}

### Local Variables: 
### tab-width: 8
### End: 
