eval 'exec perl -S $0 ${1+"$@"}'
    if 0;

# Create or extract from a tar file testing output directories which
# completely match output_version_regexp (default .* is all).

my $usage = <<"END_OF_USAGE;";
Usage: $0 [-h] [-e example:subpath:list] [-v output_version_regexp] destination
END_OF_USAGE;

use strict;
use Cwd;

# remove path from warning and error messages
$0 =~ s%^.*/%%;
BEGIN {
    $^W = 1;
    $SIG{__WARN__} = sub { warn "$0 warning: ",@_ };
}
sub dirname ($) {
    # return the directory part of a filename
    $_[0] =~ m'^(.*)/[^/]*$' ? $1 : '.';
}
# use modules in the same directory as this file if they exist.
use lib dirname __FILE__, "$ENV{CMISS_EXAMPLES}/common/cmiss_type1";
use test_cmiss;

use Getopt::Long;

my $examples_root = $ENV{CMISS_EXAMPLES} or
  die "Need to set environment variable CMISS_EXAMPLES\n";

my $output_version_regexp = ".*";
my @subpaths = ();

Getopt::Long::GetOptions(
  'h' => sub { print $usage; exit },
  'e=s' => sub { push @subpaths, split /[: ]/, $_[1] },
  #Version name pattern
  # e.g. cmgui, cmgui_optimised, cm_n32, cm_64 or cmo_n32
  'v=s' => \$output_version_regexp
 ) or die $usage;
unless( @subpaths ) { @subpaths = ('') }

@ARGV == 1 or die $usage;

my $destination = $ARGV[0];
# Make sure rsync knows this is a directory or it may create a file if there
# is only one input file.
$destination =~ m%/$% or $destination .= '/';

$SIG{__DIE__} = sub { die "$0: ",@_ };

sub exec_command (@)
  {
    my (@command) = @_;
    exec @command or die "can't exec $command[0]: $!\n";
  }

my @output_directories =
  map
  {
    my $subpath = $_;
    test_cmiss::recurse_example_tree(
      sub
	{
	  my ($examples_root,$subpath) = @_;

	  my $output_fullpath =
	    ($subpath eq '' ? $examples_root : "$examples_root/$subpath") .
	      "/output";

	  -d $output_fullpath or return ();

	  # Look for result directories based on the existence of a
	  # test summary file.

	  my @output_directories =
	    test_cmiss::recurse_directory_tree(
	      sub
		{ my ($path,$subdirs,$files) = @_;

		  if( grep { $_ eq $test_cmiss::test_summary } @$files )
		    {		# test summary exists:
		      # Don't need to look any lower
		      @$subdirs = ();
		    }
		  else
		    { # These files are of no interest, but look further down.
		      @$files = ();
		    }
		},
	      sub
		{
		  my ($path,$results,$files) = @_;

		  if( @$files )
		    {		# test_summary was found
		      return $path;
		    }
		  else
		    {
		      # Assemble list of files found below
		      return @$results;
		    }
		},
	      $output_fullpath );

	  # Filter according to $version_name_regexp;
	  @output_directories =
	    grep { m|^\Q$output_fullpath\E/$output_version_regexp$| }
	      @output_directories;

	  # Make the directory names relative to $examples_root
	  @output_directories =
	    map { s|^\Q$examples_root\E/||; $_ } @output_directories;

	  return @output_directories;

	},
      sub {1},
      sub
	{
	  # my ($examples_root, $subpath, @results) =
	  splice @_, 0, 2;
	  return @_;
	},
      $examples_root, $subpath );
  }
  @subpaths;

# There are too many directories to supply as arguments to rsync so we
# supply the list on STDIN.  This is done by forking a child to
# receive the list.

# --files-from implies --relative and removes --recursive from --archive.
# (--update might mix files from two different runs.)
my @rsync = qw(rsync
	       --verbose
	       --files-from=-
	       --archive
	       --recursive
	       --no-implied-dirs
	       --delete-after);

unless( my $pid = open LIST, '|-' )
  {
    defined $pid or die "can't fork: $!";

    # Child
    # List of directories is supplied on STDIN by the parent.
    # rsync these directories.

    exec_command(
      # --files-from implies --relative and removes --recursive from --archive.
      @rsync,
      "--exclude=$test_cmiss::test_log", $examples_root, $destination );
  }

# Parent: supply list of directories to rsync

foreach my $dir (@output_directories)
  {
    print LIST "$dir\n";
  }

close LIST or do {
  $! and die "error closing pipe from child: $!";
  $? & 0xFF and die "child received signal: $?";
  exit( $? >> 8 );
};

# Send the history files to a different location for merging.

unless( my $pid = open LIST, '|-' )
  {
    defined $pid or die "can't fork: $!";

    # Child
    # List of files is supplied on STDIN by the parent.
    # rsync these directories.

    exec_command(
      # --files-from implies --relative and removes --recursive from --archive.
      @rsync,
      $examples_root, "${destination}.histories-" . test_cmiss::hostname . '/');
  }

# Parent: supply list of files to rsync

foreach my $dir (@output_directories)
  {
    my $history_file = "$dir/$test_cmiss::test_log";
    # Check that file exists
    -f "$examples_root/$history_file" && print LIST "$history_file\n";
  }

close LIST or do {
  $! and die "error closing pipe from child: $!";
  $? & 0xFF and die "child received signal: $?";
  exit( $? >> 8 );
};
