eval 'exec perl -S $0 ${1+"$@"}'
    if 0;

# Create or extract from a tar file testing output directories which
# completely match output_version_regexp (default .* is all).

my $usage = <<"END_OF_USAGE;";
Usage: $0 -c|-x -f|--file <tarfile> [-C|--directory <examples_root>]
         [output_version_regexp]
END_OF_USAGE;

use strict;
use Cwd;

# remove path from warning and error messages
$0 =~ s%^.*/%%;
BEGIN {
    $^W = 1; # there shouldn't be any warnings
#    $SIG{__WARN__} = sub { die @_ };
    $SIG{__WARN__} = sub { warn "$0 warning: ",@_ };
}
sub dirname ($) {
    # return the directory part of a filename
    $_[0] =~ m'^(.*)/[^/]*$' ? $1 : '.';
}
# use modules in the same directory as this file if they exist.
use lib dirname __FILE__, "$ENV{CMISS_EXAMPLES}/common/cmiss_type1";
use test_cmiss;

use Getopt::Long;

my $create = 0;
my $extract = 0;
my $tarfile;
my $examples_root = $ENV{CMISS_EXAMPLES} || '.';

defined $ARGV[0] and $ARGV[0] !~ /^-/ and $ARGV[0] = "-$ARGV[0]";

Getopt::Long::config qw(bundling no_getopt_compat no_auto_abbrev);
Getopt::Long::GetOptions( 'c' => \$create,
			  'x' => \$extract,
			  'f|file=s' => \$tarfile,
			  'C|directory=s' => \$examples_root ) or die $usage;

# must specify c or x, a file, and only allow one pattern (for now).
$create ^ $extract && defined $tarfile && @ARGV <= 2 or die $usage;

my $output_version_regexp = @ARGV ? "^(?:$ARGV[0])\$" : '^';

$SIG{__DIE__} = sub { die "$0: ",@_ };

# Set up a protect mechanism that provides a finish that always
# executes and a catch for dies.

sub protect ( &% ) {
    my ($try,%ops) = @_;
    eval {
 	local $SIG{__DIE__}; # default die
	$try->();
    };
    exists $ops{finish} and do {
	# protect the finish also while retaining @_;
	my $save_at = $@;
	eval { $ops{finish}->() };
	$@ = $save_at . $@;
    };
    $@ and exists $ops{catch} ? $ops{catch}->() : die $@;
}
sub finish (&) { finish => $_[0] };
sub catch (&) { catch => $_[0] };

my $readdir_grep_string = q/^(\.\.?)$/;

sub system_or_die (@) {
    my (@command) = @_;
    #execute @command and die on failure
    system @command;
    $? != -1 or die "can't run $command[0]: $!\n";
    $? & 0xFF and die "$command[0] received signal: $?\n";
    $? == 0 or die "@command failed: ", $? >> 8, "\n";
}

if( $create ) {

    my $first_tar = 1;

    # AIX tar won't append properly to an empty tar file so we can't
    # create the tar file until we have some files.

    test_cmiss::recurse_example_tree {
	my ($examples_root,$subpath) = @_;

	my $output_fullpath =
	    ($subpath eq '' ? $examples_root : "$examples_root/$subpath") .
	    "/output";

	if( -d $output_fullpath ) {

	    opendir DIR, $output_fullpath or do {
		warn "can't opendir $output_fullpath: $!\n";
		return;
	    };

	    my @versions =
		sort { $a cmp $b } grep {
		    ! /$readdir_grep_string/o &&
			/$output_version_regexp/ &&
			-d "$output_fullpath/$_"
		    } readdir DIR;

	    closedir DIR;

	    foreach my $version (@versions) {
		my $version_subpath =
		    (length $subpath ? "$subpath/" : '') . "output/$version";

		my @command = ( 'tar', $first_tar ? 'cf' : 'rf', $tarfile,
			        '-C', $examples_root, $version_subpath );
		system_or_die @command;

		$first_tar = 0;
	    }
	}

    } sub {1}, sub {}, $examples_root, '';

    if( $first_tar ) {
	# No directories found; make an empty tar file.
	warn "no matching output directories found\n";
	my @command = (qw(tar cf), $tarfile);
	system_or_die @command;
    }
} else {

    # Get a list of the output directories in the archive.

    # Fork so that we recieve a list of files from STDOUT of tar.
    unless( my $pid = open FILE_LIST, '-|' ) {
	defined $pid or die "can't fork: $!";

	# Child:
	# Supply a list of source files on STDOUT to the parent.
	system_or_die qw(tar tf), $tarfile;
	exit;
    }

    my @tardirs;

    while( my $filename = <FILE_LIST> ) {
	chomp $filename;
	# Look for result directories based on the existence of a test_summary
	# file below an output directory.
	# Remove trailing slash as irix tar at least interprets
	# the trailing slash as do not recurse.
	if( my @matches = $filename =~
	      m"^((?:.*/)?output/(.*))/\Q$test_cmiss::test_summary\E$"o )
	  { if( $matches[1] =~ /$output_version_regexp/ )
	      { push @tardirs, $matches[0] }
	  }
    }

    close FILE_LIST or do {
	$! and die "error closing pipe from child: $!";
	$? & 0xFF and die "child received signal: $?";
	exit( $? >> 8 );
    };

    sub exec_command (@)
      {
	my (@command) = @_;
	exec @command or die "can't exec $command[0]: $!\n";
      }

    sub tar_extract ($$$)
      {
	my ($tarfile,$root,$subdir) = @_;

	# On AIX at least -C doesn't seem to work for extraction so chdir.
	$tarfile =~ m'^/' or $tarfile = getcwd . '/' . $tarfile;
	my @command = (qw(tar xf), $tarfile, $subdir);

	my $pid = fork;
	defined $pid or die "can't fork: $!\n";
	$pid or do
	  { chdir $root or die "can't chdir $root: $!\n";
	    exec_command @command;
	  };

	if( $pid != waitpid $pid, 0 )
	  { die "wait failed: $!" };

	$? & 0xFF and die "$command[0] received signal: $?\n";
	$? == 0 or die "@command failed: ", $? >> 8, "\n";
      }

    # File::Path::rmtree does not provide good error notification.
#     require File::Path;
    sub rmtree ($)
      {
	my ($path) = @_;

	test_cmiss::recurse_directory_tree
	  sub { 1 },
	    sub { my ($path,$results,$files) = @_;

		  my $result = ! grep { ! $_ } @$results;

		  foreach my $file (@$files)
		    {
		      my $filename = "$path/$file";
		      unlink $filename or do
			{
			  warn "can't unlink $filename: $!\n";
			  $result = undef;
			};
		    }

		  rmdir $path or do
		    {
		      warn "can't rmdir $path: $!\n";
		      $result = undef;
		    };

		  return $result;
		},
	      $path;
      }

    sub merge_test_logs ($$)
      {
	my ($temp_dir, $keep_dir) = @_;
	my $temp_log = "$temp_dir/$test_cmiss::test_log";
	my $keep_log = "$keep_dir/$test_cmiss::test_log";

	return if ! -e $temp_log;

	if( ! -f $keep_log )
	  { rename $temp_log, $keep_log
	      or die "can't rename $temp_log, $keep_log: $!\n";
	    return;
	  }

	open KEEP, "+< $keep_log" or die "can't open +< $keep_log: $!\n";
	open TEMP, "< $temp_log" or die "can't open < $temp_log: $!\n";

	# skip header lines
	scalar <KEEP>;
	scalar <TEMP>;

	# Search for the first place where lines of $temp_log are not in
	# $keep_log.

	my ($temp_line,$keep_line,$write_pos);
	my $cmp = 0; # take a new line from each file

	do { $write_pos = tell KEEP;
	     $keep_line = <KEEP>;
	     if( $cmp == 0 ) # lines are the same
	       { $temp_line = <TEMP> };
	   }
	  while( defined $keep_line and defined $temp_line
		   and # keep_line is older or same as temp_line
		     ($cmp = $keep_line cmp $temp_line) <= 0 );

	# Record the lines that need to be merged.
	my @newlines;

	while( defined $keep_line or defined $temp_line )
	  {
	    my $cmp = do
	      { if( ! defined $keep_line )
		  { 1 } # take temp line
		elsif( ! defined $temp_line )
		  { -1 } # take keep_line
		else
		  { $keep_line cmp $temp_line }
	      };

	    if( $cmp <= 0 )
	      { # keep_line is older (or no temp_line) or same as temp_line
		push @newlines, $keep_line;
		$keep_line = <KEEP>;
		if( $cmp == 0 ) # lines are the same
		  { $temp_line = <TEMP> };
	      }
	    else
	      { # temp_line older (or no keep_line)
		push @newlines, $temp_line;
		$temp_line = <TEMP>;
	      }
	  }

	seek KEEP, $write_pos, 0 or die "seek failed on $keep_log\n";

	print KEEP @newlines;

	close KEEP or die "close failed on $keep_log: $!\n";
	close TEMP;
      }
    # Choose a directory in which to work.  If this is on the same filesystem
    # as the $examples_root, then moving files is simply renaming.
    my $work_root =
      "$examples_root/tar_output." . &test_cmiss::hostname() . ".$$";

    mkdir $work_root or die "can't mkdir $work_root: $!\n";

    foreach my $subdir (@tardirs)
      {
	my $dir = "$examples_root/$subdir";

	protect
	  {
	    local $SIG{INT} = sub
	      {
		die "interrupt received\n";
	      };
	    local $SIG{TERM} = sub
	      {
		die "SIGTERM received\n";
	      };

	    test_cmiss::start_lock_output $dir ;

	    if( -d $dir )
	      {	# Extract from tar to temporary location
		tar_extract $tarfile, $work_root, $subdir;

		my $work_dir = "$work_root/$subdir";

		# Replace all files except the history if the test from the
		# tar is newer.
		my $temp_summary = "$work_dir/$test_cmiss::test_summary";
		my $test_summary = "$dir/$test_cmiss::test_summary";

		# $temp_summary should exist as its presence in the tar file
		# was used in selecting this directory.
		-f $temp_summary or die "no file $temp_summary";
		if( ! -f $test_summary or
		      -M $temp_summary < -M $test_summary )
		  {
		    opendir DIR, $dir or
		      die "can't opendir $dir: $!\n";

		    my @contents = grep
		      { ! /$readdir_grep_string/o &&
			  $_ ne $test_cmiss::test_log } readdir DIR;

		    closedir DIR;

		    foreach my $item (@contents)
		      {
			$item = "$dir/$item";
			if( -d $item )
			  { rmtree $item or die "failed to remove $item\n" }
			else
			  { unlink $item or die "can't unlink $item: $!\n" }
		      }

		    opendir DIR, $work_dir or
		      die "can't opendir $work_dir: $!\n";

		    @contents = grep
		      { ! /$readdir_grep_string/o &&
			  $_ ne $test_cmiss::test_log } readdir DIR;

		    closedir DIR;

		    foreach my $item (@contents)
		      {
			my $source = "$work_dir/$item";
			my $destination = "$dir/$item";

			rename $source, $destination
			  or die "can't rename $source, $destination: $!\n";
		      }
		  }

		# merge the new test_log into the old.
		merge_test_logs $work_dir, $dir;

		# Remove temporary tree
		$subdir =~ m%^([^/]*)%; # get top directory
		rmtree "$work_root/$1";
	      }
	    else
	      {	 # No directory to merge.  Extract directly to final location.
		tar_extract $tarfile, $examples_root, $subdir;
	      }

	    test_cmiss::end_lock_output $dir;
	  }
	    catch
	      {
		warn $@;
		print STDERR "cleaning up locks...";
		&test_cmiss::abort_locks();
		die "done\n";
	      };
      }

    rmdir $work_root or die "can't rmdir $work_root: $!\n";
}
