#!/usr/bin/perl -w
use strict;
#use diagnostics;

use Getopt::Long;

use lib "$ENV{CMISS_EXAMPLES}/common/cmiss_type1";
use test_cmiss;

my $usage = "Usage: $0 [-h] -e example -s operating_system -x executable [-c compare_version] [-o output_directory] [-k] [Other flags for test_run]\n"; 
# This script tests an example using cm in the same way as the pre2000 scripts
# used to.

#Set defaults
my $example_path = "";
my $operating_system;
my $executable = 0;
my $compare_version = "generic";
my $output_dir = 0;
my $keep_all_output = 0;
my %options;
my $verbose = 0;

#Keep all the flags so we can pass them along.
my @args = @ARGV;

if (@ARGV < 1)
  {
    print $usage;
    exit;
  }
#Process the command line
Getopt::Long::config qw(bundling no_getopt_compat no_auto_abbrev pass_through);
unless( Getopt::Long::GetOptions
	('h' => sub { print $usage; exit },
	 #Verbose
	 'verbose' => sub { $verbose = 1 },
	 #Operating system pattern i.e. irix, linux or irix64
	 's=s' => \$operating_system,
	 #The relative path to this example i.e. a/a1
	 'e=s' => \$example_path,
	 #Executable i.e. /product/cmiss/bin/cmgui or my_cmo_n32
	 #If executable is specified then this overrides that specified in the versions file.
	 'x=s' => \$executable,
	 #Compare_version i.e. generic, cmgui_optimised or hpc
	 #If compare_version is specified then this overrides that specified in the versions file.
	 'c=s' => \$compare_version,
	 #Keep all the output in the output directories
	 'k' => sub { $keep_all_output = 1 },
	 #Result path, either full path (i.e. /usr/tmp/output) or relative to this example
	 #(i.e. cmgui_linux will become /product/cmiss/examples/1/13/133/output/cmgui_linux)
	 'o=s' => \$output_dir
	 ) ) {
    print STDERR $usage;
    exit 1;
}

if (! $example_path)
  {
    print "Example option missing\n";
    print $usage;
    exit;
  }
if (! $executable)
  {
    print "Executable option missing\n";
    print $usage;
    exit;
  }
unless( defined $operating_system )
  {
    print "Operating system option missing\n";
    print $usage;
    exit;
  }

sub abort {
    die "$0: ",@_;
}
sub mywarn {
    warn "$0 warning: ",@_;
}

$SIG{__DIE__} = \&abort;
$SIG{__WARN__} = \&mywarn;

#Where all the examples are stored on disk i.e. /product/cmiss/examples
my $example_location = $ENV{CMISS_EXAMPLES};
#Script/template path
my $template_path = "$example_location/common/cmiss_type1";

my $example_dir = $example_location . "/" . $example_path;
my $example_name = substr ($example_dir, rindex ($example_dir, '/') + 1);
my $result_path = ();
my $result_lock_file = ();
if ($output_dir)
  {
    if ($output_dir =~ m%^/%)
      {
	$result_path = $output_dir;
      }
    else
      {
	$result_path = "$example_dir/output/$output_dir";
      }
  }
my $test_output = "cmiss_test.log";
my $pid;

my @diff_program_paths =
    map {"$template_path/$_"} ("${operating_system}_bin",'bin');
my $readdir_grep_string = q/^(\.|CVS)/;

#Check that the current directory is empty.
opendir(DIR, ".") || die "can't opendir current working directory: $!\n";
my @current_dir = grep { ! /$readdir_grep_string/ } readdir(DIR);
closedir DIR;
if (@current_dir)
{
#  die "Current working directory is not empty";
    warn "Current working directory is not empty\n";
}

# Do the execution in a forked process and watch with the parent.

my $test_run = "$example_dir/control/test_run";
my @command = ($test_run,@args,'-l',$test_output);

my $start_time = time();
if ($verbose)
  {
    print "Start time: " . localtime($start_time) . "\n";
  }

-1 == system @command and die "can't run $test_run: $!\n";

my $end_time = time();
# times for subprocess (assuming no other subprocesses)
my @cpu_times = (times)[2,3];
my $wait_status = $?;
if ($verbose)
  {
    print "Completed execution: " . localtime($end_time) . "\n";
  }
my $clock_time = $end_time - $start_time;

my ($failed_output_ref, $passed_output_ref) = 
  test_cmiss::compare_files($example_dir, $compare_version,
			    \@diff_program_paths, $wait_status, $verbose);

test_cmiss::generate_summary( $example_dir, $example_path, $compare_version,
			      $result_path,
			      $failed_output_ref, $passed_output_ref,
			      $start_time, $clock_time,
			      \@cpu_times, $keep_all_output, $verbose );

if (@$failed_output_ref)
  {
    exit (1);
  }
else
  {
    exit (0);
  }

### Local Variables: 
### tab-width: 8
### End: 
