eval 'exec perl -wS $0 ${1+"$@"}'
    if 0;

use strict;
use File::Basename;
use Getopt::Long;

my $usage = "Usage: $0 [-h] [--verbose] -e example [-s operating_system] -x cm_executable -l stdouterr_filename [-p number_of_processes] [--clear-malloc] [--set \"values\"]\n";

my $example_path;
my $operating_system;
my $log_file;
my $executable;
my $nprocs;
my $presets;
my $verbose = 0;

# !!! KAT: we are not going to pass through any options to anything
# but test.pl gives us some extra stuff we don't want.  We then can't
# auto_abbrev as a -c could be interpreted as a clear-malloc.
# pass_through, no_auto_abbrev should be removed and test.pl fixed.

Getopt::Long::Configure qw(bundling no_getopt_compat pass_through no_auto_abbrev);
unless( Getopt::Long::GetOptions
	('h' => sub { print $usage; exit },
	 #Verbose
	 'verbose' => sub { $verbose = 1 },
	 #Operating system pattern i.e. irix, linux or irix64
	 's=s' => \$operating_system,
	 #The relative path to this example i.e. a/a1
	 'e=s' => \$example_path,
	 #Log file.
	 'l=s' => \$log_file,
	 #Executable i.e. /product/cmiss/bin/cm/cm_n32 or my_cmo_n32
	 'x=s' => \$executable,
	 #Number of threads
	 'p=i' => \$nprocs,
	 #CM Presets
	 'set=s' => \$presets,
	 #Initialized malloced memory
	 # Should the pattern be random perhaps: this would provide more
	 # thorough testing but make it difficult to reproduce results.
	 # Using just two different values for now.
	 #  -370086 as an integer*4, segmentation violation as a
	 #  pointer, a (probably quiet) NaN as a real, and
	 #  a (probably signaling) NaN as a double.
	 'clear-malloc' => sub
	   {
	     $ENV{_SSMALLOC_FASTCHK} = '';
	     $ENV{_SSMALLOC_CLEAR_MALLOC} = '';
	     $ENV{_SSMALLOC_CLEAR_MALLOC_PATTERN} = '0xfffa5a5a';
	   },
	 #  2146959359 as an integer*4, bus error as a pointer to anything
	 #  that should be aligned on an even address, a (probably quiet) NaN
	 #  as a real, and a (probably signaling) NaN as a double.
	 'clear-malloc7' => sub
	   {
	     $ENV{_SSMALLOC_FASTCHK} = '';
	     $ENV{_SSMALLOC_CLEAR_MALLOC} = '';
	     $ENV{_SSMALLOC_CLEAR_MALLOC_PATTERN} = '0x7ff7ffff';
	   } )
	) {
    print STDERR $usage;
    exit 1;
}

unless( defined $example_path )
{
    print STDERR "-e example option missing\n";
    print STDERR $usage;
    exit 1;
}
unless( defined $log_file )
{
    print STDERR "-l stdouterr_filename option missing\n";
    print STDERR $usage;
    exit 1;
}
unless( defined $executable )
{
    print STDERR "-x cm_executable option missing\n";
    print STDERR $usage;
    exit 1;
}

unless( defined $operating_system )
  {
    require POSIX;
    $operating_system = (&POSIX::uname())[0];
  }

sub abort {
    die "$0: ",@_;
}
sub mywarn {
    warn "$0 warning: ",@_;
}

$SIG{__DIE__} = \&abort;
$SIG{__WARN__} = \&mywarn;

my $example_location = $ENV{CMISS_EXAMPLES};
my $template_path ="$example_location/common/cmiss_type1";
my $example_name=basename($example_path);
my $example_comfile;
# Try to use resolve_example_path to find the command file to use
my $resolve = "$example_location/common/resolve_example_path";
# Should trim NUL from end of output from $resolve.
my @resolved = split ' ',`$resolve $example_path`;
if (@resolved == 2) {
    $example_comfile = $resolved[1];
} else {
    # If resolve_example_path didn't work, just assume things will work.
    if (-f "$example_location/$example_path/cmiss_input/example_$example_name.com") {
        $example_comfile = "example_$example_name";
    } else {
        $example_comfile = "$example_name";
    }
}
my $temp_comfile_name = "cm_test_$example_name.com";

# make sure DISPLAY environment variable is not set
delete $ENV{DISPLAY};

open(TEMP_COM,">$temp_comfile_name") || die "Error opening file, $!\n";

# Need to quote the example_path to avoid paths like 1/11/113
# being treated as a series of divisions.
print TEMP_COM "set directory example \"$example_path\"\n";

# LKC 21-NOV-2001 $TESTING env variable as per test_run_cmgui.pl
#
# Set the $TESTING flag to 1 in the command file. Comfiles may put condition
# "if ($TESTING) ..." around commands they only want to run during testing.
print TEMP_COM "\$TESTING = 1;\n";

# rgb 16-10-2001 - setting the number of processes for the example
if( defined $nprocs ) {
    # Force use of the maximum number of threads
    $ENV{OMP_DYNAMIC}='FALSE';
    print TEMP_COM "set num_threads $nprocs\n";
}
if( defined $presets ) {
    printf TEMP_COM "%s\n", $presets;
}

print TEMP_COM "read command;\$example/$example_comfile\n";

if (-f "$example_location/$example_path/cmiss_input/test_output.com")
{
    print TEMP_COM "set output;cmiss_test on\n";
    # This would add a line to the cmiss_test.out file
#     print TEMP_COM "read command;\$example/test_output.com\n";
    open(PART,"<$example_location/$example_path/cmiss_input/test_output.com") || die "Error opening test_output.com, $!\n";
    print TEMP_COM $_ while( <PART>);
    close(PART) || die;
    print TEMP_COM "set output off\n";
}

close(TEMP_COM) || die "error closing >$temp_comfile_name: $!\n";
my @command = ($executable,'-batch',$temp_comfile_name);

if ($verbose)
  {
    print "Executing @command\n";
  }

if( $operating_system =~ /^irix(64)?$/i )
  {
    # On IRIX rld errors are usually sent to /dev/tty or /var/adm/SYSLOG.
    # Redirect them to the log file.
    $ENV{_RLD_ARGS} = "-log $log_file";
  }

if( defined fileno STDIN ) {
    open STDIN, '</dev/null' or warn "error opening </dev/null: $!\n";
}
# Opening $log_file as ">$log_file" overwrites the rld errors,
# so unlink any old file then open to append.
! -f $log_file or unlink $log_file or die "can't unlink $log_file: $!\n";
open STDOUT, ">>$log_file" or die "can't open >$log_file: $!\n";
open STDERR, ">&STDOUT" or die "can't dup STDOUT: $!\n";

exec @command or
    die "can't exec @command: $!\n";

### Local Variables: 
### tab-width: 8
### End: 
